Claw = {}

if select(2, UnitClass("player")) ~= "DRUID" then
    DisableAddOn("Claw")
    return
end

SLASH_Claw1, SLASH_Claw2 = "/claw", "/clawesome"

local function InitializeVariables()
	Claw.locked = Claw.locked == nil and false or Claw.locked
	Claw.scale = Claw.scale == nil and 1 or Claw.scale
	Claw.frequency = Claw.frequency == nil and 0.05 or Claw.frequency
	Claw.latency = Claw.latency == nil and 0.3 or Claw.latency
	Claw.previous = Claw.previous == nil and true or Claw.previous
	Claw.cd = Claw.cd == nil and true or Claw.cd
	Claw.misseffect = Claw.misseffect == nil and true or Claw.misseffect
	Claw.glow = Claw.glow == nil and true or Claw.glow
	Claw.cat = Claw.cat == nil and true or Claw.cat
	Claw.bear = Claw.bear == nil and true or Claw.bear
	Claw.bossonly = Claw.bossonly == nil and false or Claw.bossonly
	Claw.ff = Claw.ff == nil and true or Claw.ff
	Claw.tfberserk = Claw.tfberserk == nil and false or Claw.tfberserk
	Claw.motw = Claw.motw == nil and false or Claw.motw
	Claw.front = Claw.front == nil and false or Claw.front
end

local events, targetHealthArray = {}, {}
local lastAbilityCalculation, currentTime, ripExtendsLeft, lastRakeTickDamage, lastRipTickDamage = 0, 0, 0, 0, 0
local hostileTargetted, playerHasGlyphOfSavagery = false, false, false

local clawPanel = CreateFrame("Frame", "clawPanel", UIParent)
clawPanel:SetPoint("CENTER", 0, -169)
clawPanel:SetFrameStrata("BACKGROUND")
clawPanel:Hide()
clawPanel:RegisterForDrag("LeftButton")
clawPanel:SetScript("OnDragStart", clawPanel.StartMoving)
clawPanel:SetScript("OnDragStop", clawPanel.StopMovingOrSizing)
clawPanel:SetMovable(true)
clawPanel.icon = clawPanel:CreateTexture(nil, "BACKGROUND")
clawPanel.icon:SetAllPoints(clawPanel)
clawPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPanel.border = clawPanel:CreateTexture(nil, "BORDER")
clawPanel.border:SetAllPoints(clawPanel)
clawPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
clawPanel.border:Hide()
local clawPreviousPanel = CreateFrame("Frame", "clawPreviousPanel", UIParent)
clawPreviousPanel:SetPoint("BOTTOMRIGHT", clawPanel, "BOTTOMLEFT", -10, -5)
clawPreviousPanel:SetFrameStrata("BACKGROUND")
clawPreviousPanel:Hide()
clawPreviousPanel:RegisterForDrag("LeftButton")
clawPreviousPanel:SetScript("OnDragStart", clawPreviousPanel.StartMoving)
clawPreviousPanel:SetScript("OnDragStop", clawPreviousPanel.StopMovingOrSizing)
clawPreviousPanel:SetMovable(true)
clawPreviousPanel.icon = clawPreviousPanel:CreateTexture(nil, "BACKGROUND")
clawPreviousPanel.icon:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPreviousPanel.border = clawPreviousPanel:CreateTexture(nil, "BORDER")
clawPreviousPanel.border:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
local clawCDPanel = CreateFrame("Frame", "clawCDPanel", UIParent)
clawCDPanel:SetPoint("BOTTOMLEFT", clawPanel, "BOTTOMRIGHT", 10, -5)
clawCDPanel:SetFrameStrata("BACKGROUND")
clawCDPanel:Hide()
clawCDPanel:RegisterForDrag("LeftButton")
clawCDPanel:SetScript("OnDragStart", clawCDPanel.StartMoving)
clawCDPanel:SetScript("OnDragStop", clawCDPanel.StopMovingOrSizing)
clawCDPanel:SetMovable(true)
clawCDPanel.icon = clawCDPanel:CreateTexture(nil, "BACKGROUND")
clawCDPanel.icon:SetAllPoints(clawCDPanel)
clawCDPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawCDPanel.border = clawCDPanel:CreateTexture(nil, "BORDER")
clawCDPanel.border:SetAllPoints(clawCDPanel)
clawCDPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")

local Ability, abilityList = {}, {}
Ability.__index = Ability

function Ability.add(spellId, buff, playerCast)
    local ability = {}
    setmetatable(ability, Ability)
	ability.spellId = spellId
    ability.buff = buff or false
    ability.playerCast = playerCast or false
    ability.name = GetSpellInfo(spellId)
    ability.icon = select(3, GetSpellInfo(spellId))
    return ability
end

function Ability:ready(seconds)
	return self:cooldown() <= (seconds or 0)
end

function Ability:getAura()
	for i = 1, 40 do
		if select(11, UnitAura(self.buff and "player" or "target", i, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))) == self.spellId then
			return UnitAura(self.buff and "player" or "target", i, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))
		end
	end
end
function Ability:remains()
	return max((select(7, self:getAura()) or 0) - currentTime - Claw.latency, 0)
end

function Ability:up()
    return select(6, self:getAura()) == 0 or self:remains() > 0
end

function Ability:down()
	return not self:up()
end

function Ability:cooldown()
    local startTime, duration = GetSpellCooldown(self.spellId)
    return max(duration - (currentTime - startTime) - Claw.latency, 0)
end

function Ability:stack()
	return select(4, self:getAura()) or 0   
end

function Ability:inRange()
    return IsSpellInRange(self.name, "target") == 1
end

function Ability:known()
	return GetSpellBookItemInfo(self.name) and true or false
end

function Ability:checkButtonAndGlow(button, on)
	if button then
		local actionType, spellIdOrMacro = GetActionInfo(button.action)
		if actionType then
			if (actionType == "spell" and spellIdOrMacro == self.spellId) or (actionType == "macro" and (select(2, GetMacroInfo(spellIdOrMacro))) == self.icon) then
				if on then
					ActionButton_ShowOverlayGlow(button)
				else
					ActionButton_HideOverlayGlow(button)
				end
			end
		end
	end
end

function Ability:glow(on)
	for i = 1, 12 do
		self:checkButtonAndGlow(_G["ActionButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarRightButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomRightButton" .. i], on)
	end
	if Dominos then
		for i = 1, 60 do self:checkButtonAndGlow(_G["DominosActionButton" .. i], on) end
	end
end

-- Druid abilities
local BerserkBear = Ability.add(50334, true, true)
local BerserkCat = Ability.add(106951, true, true)
local FaerieFire = Ability.add(770, false, false)
local OmenOfClarity = Ability.add(135700, true, true)
local DreamOfCenarius = Ability.add(108373, true, true)
local DreamOfCenariusDamage = Ability.add(108381, true, true)
local NaturesVigil = Ability.add(124974, true, true)
local NaturesSwiftness = Ability.add(132158, true, true)
local PredatorySwiftness = Ability.add(69369, true, true)
local IncarnationFeral = Ability.add(102543, true, true)
local SoulOfTheForest = Ability.add(114107, true, true)
local HealingTouch = Ability.add(5185, true, true)
local ForceOfNature = Ability.add(106737, false, true)
-- Cat abilities
local CatForm = Ability.add(768, true, true)
local FerociousBite = Ability.add(22568, false, true)
local MangleCat = Ability.add(33876, false, false)
local Prowl = Ability.add(5215, true, true)
local Rake = Ability.add(1822, false, true)
local Ravage = Ability.add(6785, false, true)
local Rip = Ability.add(1079, false, true)
local SavageRoarCP = Ability.add(52610, true, true)
local SavageRoarGlyph = Ability.add(127538, true, true)
local SavageRoar = Ability.add(127538, true, true)
local Shred = Ability.add(5221, false, true)
local TigersFury = Ability.add(5217, true, true)
local ThrashCat = Ability.add(106830, false, true)
-- Bear abilities
local BearForm = Ability.add(5487, true, true)
local Enrage = Ability.add(5229, true, true)
local Lacerate = Ability.add(33745, false, true)
local MangleBear = Ability.add(33878, false, false)
local Maul = Ability.add(6807, false, true)
local SwipeBear = Ability.add(779, false, true)
local Thrash = Ability.add(77758, false, true)
--   10% stat buff
local BlessingOfKings = Ability.add(20217, true)
local EmbraceOfTheShaleSpider = Ability.add(90363, true)
local LegacyOfTheEmperor = Ability.add(117666, true)
local MarkOfTheWild = Ability.add(1126, true)
--   12% armor reduction
local WeakenedArmor = Ability.add(113746)
--   Potions
local VirmensBite = Ability.add(105697, true, true)
-- Non-existent spell
local NoSpell = Ability.add(0)

local abilityNow, oldAbilityNow, abilityPrevious, abilityCD = NoSpell, NoSpell, NoSpell, NoSpell

function SavageRoar:up()
	return SavageRoarCP:up() or SavageRoarGlyph:up()
end

function SavageRoar:remains()
	return SavageRoarCP:up() and SavageRoarCP:remains() or SavageRoarGlyph:remains()
end

local function ComboPoints()
    return GetComboPoints("player", "target")
end

local function Energy()
    return CatForm:up() and min(UnitPower("player") + (GetPowerRegen() * Claw.latency), 100) or 100
end

local function EnergyTimeToCap()
	return (100 - Energy()) / GetPowerRegen()
end

local function Rage()
    return BearForm:up() and UnitPower("player") or 0
end

local function TargetHealthPercentage()
    return UnitHealth("target") / UnitHealthMax("target") * 100
end

local function TargetTimeToDie()
    local targetTimeToDie = UnitHealth("target") / ((targetHealthArray[1] - targetHealthArray[#targetHealthArray]) / 3) - Claw.latency
    return targetTimeToDie >= 0 and targetTimeToDie or 600
end

local function TargetIsBoss()
	return UnitLevel("target") == -1 or (UnitLevel("target") >= UnitLevel("player") + 2 and not UnitInRaid("player"))
end

local function StatBuffActive()
	return BlessingOfKings:up() or EmbraceOfTheShaleSpider:up() or LegacyOfTheEmperor:up() or MarkOfTheWild:up()
end

local function CrazyRipRemainsMath()
	return Rip:remains() + (8 - ((3 - ripExtendsLeft) * 2))
end

local function GetCurrentDamageMultiplier()
	return select(7, UnitDamage("player")) + (DreamOfCenariusDamage:up() and 0.25 or 0)
end

local function ModifyForCrit(damage)
	return damage + damage * (min(100, GetCritChance()) / 100) * 1.03
end

local function GetNextRakeTickDamage()
	return ModifyForCrit((118 + (0.368 * UnitAttackPower("player"))) * (1 + GetMasteryEffect() / 100) * 0.8154868 * GetCurrentDamageMultiplier())
end

local function GetNextRipTickDamage()
	return ModifyForCrit(((113 + 320 * ComboPoints()) + (0.0484 * ComboPoints() * UnitAttackPower("player"))) * (1 + GetMasteryEffect() / 100) * GetCurrentDamageMultiplier())
end

local function DiffersPos(num1, num2, percent)
	return (num2 - num1) / num1 * 100 > (percent or 0)
end

local function CalculateBestChoiceAbilityCat()
	if Claw.cd and (not Claw.bossonly or TargetIsBoss()) then
		if Claw.tfberserk and BerserkCat:ready() and ((TigersFury:ready() and Energy() <= 35 and OmenOfClarity:down()) or (TargetTimeToDie() <= 15 and not TigersFury:ready(6))) then
			abilityCD = BerserkCat
		elseif TigersFury:ready() and Energy() <= 35 and BerserkCat:down() and OmenOfClarity:down() then
			abilityCD = TigersFury
		elseif BerserkCat:ready() and (TigersFury:up() or (TargetTimeToDie() <= 15 and not TigersFury:ready(6))) then
			abilityCD = BerserkCat
		elseif NaturesVigil:known() and NaturesVigil:ready() and BerserkCat:up() then
			abilityCD = NaturesVigil
		elseif IncarnationFeral:known() and IncarnationFeral:ready() and BerserkCat:up() then
			abilityCD = IncarnationFeral
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if Claw.motw and not StatBuffActive() then
        return MarkOfTheWild
	elseif not InCombatLockdown() and DreamOfCenarius:known() and DreamOfCenariusDamage:down() then
		return HealingTouch
	elseif not InCombatLockdown() and CatForm:down() then
		return CatForm
	elseif not InCombatLockdown() and SavageRoar:remains() < 6 and (ComboPoints() ~= 0 or playerHasGlyphOfSavagery) then
		return SavageRoar
	elseif not InCombatLockdown() and ForceOfNature:known() and ForceOfNature:ready() then
		return ForceOfNature
    elseif Claw.ff and FaerieFire:ready() and WeakenedArmor:stack() ~= 3 then
        return FaerieFire
	elseif (ComboPoints() ~= 0 or playerHasGlyphOfSavagery) and SavageRoar:down() then
		return SavageRoar
	elseif DreamOfCenarius:known() and (NaturesSwiftness:up() or (PredatorySwiftness:remains() > 0.2 and ((ComboPoints() >= 4 and DreamOfCenariusDamage:stack() < 2) or (PredatorySwiftness:remains() > 0.2 and PredatorySwiftness:remains() <= max(1, (30 - Energy()) / GetPowerRegen()) and DreamOfCenariusDamage:down())))) then
		return HealingTouch
	elseif ComboPoints() ~= 0 and Rip:up() and Rip:remains() <= 2 and TargetHealthPercentage() <= 25 then
		return FerociousBite
	elseif OmenOfClarity:up() and ThrashCat:remains() < 3 and DreamOfCenariusDamage:down() then
		return ThrashCat
	elseif (ComboPoints() ~= 0 or playerHasGlyphOfSavagery) and TargetHealthPercentage() <= 25 and (SavageRoar:remains() <= 1 or (SavageRoar:remains() <= 3 and ComboPoints() ~= 0)) then
		return SavageRoar
	elseif NaturesSwiftness:known() and DreamOfCenarius:known() and NaturesSwiftness:ready() and ComboPoints() == 5 and DreamOfCenariusDamage:down() and PredatorySwiftness:down() and TargetHealthPercentage() <= 25 then
		return NaturesSwiftness
	elseif ComboPoints == 5 and DreamOfCenariusDamage:up() and DiffersPos(lastRipTickDamage, GetNextRipTickDamage(), 5) and TargetHealthPercentage() <= 25 and TargetTimeToDie() > 30 then
		return Rip
	elseif ComboPoints() == 5 and Rip:up() and TargetHealthPercentage() <= 25 then
		return FerociousBite
	elseif ComboPoints() == 5 and TargetTimeToDie() >= 6 and ((Rip:remains() <= 2 and DreamOfCenariusDamage:up()) or (Rip:remains() <= 6 and DreamOfCenariusDamage:up() and DiffersPos(lastRipTickDamage, GetNextRipTickDamage(), 5) and TargetHealthPercentage() > 25)) then
		return Rip
	elseif (ComboPoints() ~= 0 or playerHasGlyphOfSavagery) and (SavageRoar:remains() <= 1 or (SavageRoar:remains() <= 3 and ComboPoints() ~= 0)) then
		return SavageRoar
	elseif NaturesSwiftness:known() and DreamOfCenarius:known() and NaturesSwiftness:ready() and ComboPoints() == 5 and DreamOfCenariusDamage:down() and PredatorySwiftness:down() and TargetHealthPercentage() > 25 and (Rip:remains() < 3 and (BerserkCat:up() or Rip:remains() <= TigersFury:cooldown())) then
		return NaturesSwiftness
	elseif ComboPoints() == 5 and TargetTimeToDie() >= 6 and Rip:remains() <= 2 and (BerserkCat:up() or Rip:remains() <= TigersFury:cooldown()) then
		return Rip
	elseif OmenOfClarity:up() and ThrashCat:remains() <= 3 then
		return ThrashCat
	elseif ripExtendsLeft >= 1 and Rip:up() and Rip:remains() <= 4 then
		return IncarnationFeral:up() and Ravage or (Claw.front and MangleCat or Shred)
	elseif (ComboPoints() ~= 0 and TargetTimeToDie() <= 1) or (TargetTimeToDie() <= 4 and ComboPoints() == 5) then
		return FerociousBite
	elseif ComboPoints() == 5 and Rip:up() and ((CrazyRipRemainsMath() > 6 and (SoulOfTheForest:known() or BerserkCat:up())) or CrazyRipRemainsMath() > 10) then
		return SavageRoar:remains() <= 6 and SavageRoar or FerociousBite
	elseif TargetTimeToDie() >= 8.5 and ((DreamOfCenariusDamage:up() and DiffersPos(lastRakeTickDamage, GetNextRakeTickDamage(), 15)) or (Rake:remains() <= 3 and (BerserkCat:up() or TigersFury:cooldown() + 0.8 >= Rake:remains()))) then
		return Rake
	elseif OmenOfClarity:up() or (PredatorySwiftness:remains() > 1 and not (Energy() + (GetPowerRegen() * (PredatorySwiftness:remains() - 1)) < (4 - ComboPoints()) * 20)) or (ComboPoints() < 5 and Rip:remains() <= 3) or (ComboPoints() == 0 and SavageRoar:remains() < 2) or TargetTimeToDie() <= 8.5 then
		return IncarnationFeral:up() and Ravage or (Claw.front and MangleCat or Shred)
	elseif ComboPoints() == 5 and ThrashCat:remains() < 6 and (TigersFury:up() or BerserkCat:up() or TigersFury:ready(3) or Energy() >= 85) then
		return ThrashCat
	elseif TigersFury:up() or BerserkCat:up() or TigersFury:ready(3) or Energy() >= 85 then
		return IncarnationFeral:up() and Ravage or (Claw.front and MangleCat or Shred)
	elseif ForceOfNature:known() and ForceOfNature:ready() then
		return ForceOfNature
	else
		return NoSpell
	end
end

local function CalculateBestChoiceAbilityBear()
	if Claw.cd and (not Claw.bossonly or TargetIsBoss()) then
		if Maul:ready() and Rage() >= 50 then
			abilityCD = Maul
		elseif Enrage:ready() and Rage() <= 30 then
			abilityCD = Enrage
		else
			abilityCD = NoSpell
		end
	else
		abilityCD = NoSpell
	end
	if Claw.motw and not StatBuffActive() then
        return MarkOfTheWild
	elseif not InCombatLockdown() and BearForm:down() then
		return BearForm
	elseif not MangleBear:inRange() and FaerieFire:ready(0.4) and WeakenedArmor:stack() ~= 3 then
        return FaerieFire
	elseif MangleBear:ready(0.4) then
		return MangleBear
	elseif Thrash:ready(0.4) and Thrash:remains() <= 2 then
		return Thrash
	elseif Lacerate:ready(0.4) and Lacerate:stack() ~= 3 then
		return Lacerate
    elseif FaerieFire:ready(0.4) and WeakenedArmor:stack() ~= 3 then
        return FaerieFire
	elseif Lacerate:ready(0.4) then
        return Lacerate
	elseif FaerieFire:ready(0.4) then
		return FaerieFire
	elseif SwipeBear:ready(0.4) then
		return SwipeBear
	else
		return NoSpell
	end
end

local function CalculateBestChoiceAbility()
	if Claw.cat and BearForm:down() and GetSpecialization() == 2 then
		return CalculateBestChoiceAbilityCat()
	elseif Claw.bear and CatForm:down() and GetSpecialization() == 3 then
		return CalculateBestChoiceAbilityBear()
	else
		abilityCD = NoSpell
		clawPreviousPanel:Hide()
		return NoSpell
	end
end

function events:PLAYER_TARGET_CHANGED()
	hostileTargetted = not UnitCanAttack("player", "target")
end

function events:ACTIVE_TALENT_GROUP_CHANGED()
	playerHasGlyphOfSavagery = false
	for i = 1, GetNumGlyphSockets() do
		if select(4, GetGlyphSocketInfo(i)) == 127540 then
			playerHasGlyphOfSavagery = true
			break
		end
	end
end

function events:PLAYER_ENTERING_WORLD()
	events:ACTIVE_TALENT_GROUP_CHANGED()
end

function events:ADDON_LOADED(name)
	if name == "Claw" then
		if not Claw.scale then
			print("It looks like this is your first time running Claw, why don't you take some time to familiarize yourself with the commands?")
			print("Type |cFFFFD000/claw|r for a list of commands.")
		end
		InitializeVariables()
		clawPanel:SetSize(64, 64)
		clawPanel:SetScale(Claw.scale)
		clawPanel:EnableMouse(not Claw.locked)
		clawPreviousPanel:SetSize(42, 42)
		clawPreviousPanel:SetScale(Claw.scale)
		clawPreviousPanel:EnableMouse(not Claw.locked)
		clawCDPanel:SetSize(42, 42)
		clawCDPanel:SetScale(Claw.scale)
		clawCDPanel:EnableMouse(not Claw.locked)
        for i = 1, math.floor(3 / Claw.frequency) do targetHealthArray[i] = 0 end
		clawPanel:Show()
	end
end

function events:UNIT_SPELLCAST_SENT(srcName, spellName)
	if srcName == "player" then
		if Claw.previous and spellName == abilityNow.name then
			abilityPrevious = abilityNow
		end
		if spellName == "Rake" then
			lastRakeTickDamage = GetNextRakeTickDamage()
		elseif spellName == "Rip" then
			lastRipTickDamage = GetNextRipTickDamage()
		end
	end
end

function events:COMBAT_LOG_EVENT_UNFILTERED(self, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId, spellName, spellSchool, auraType)
	if Claw.previous then
		if eventType == "SPELL_CAST_SUCCESS" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			clawPreviousPanel.icon:SetTexture(abilityPrevious.icon)
			clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
			clawPreviousPanel:Show()
		elseif Claw.misseffect and eventType == "SPELL_MISSED" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\misseffect.blp")
		end
	end
    if srcGUID == UnitGUID("player") and dstGUID == UnitGUID("target") then
        if spellId == Rip.spellId and (eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH") then
            ripExtendsLeft = 3
        elseif (spellId == Shred.spellId or spellId == MangleCat.spellId or spellId == Ravage.spellId) and eventType == "SPELL_DAMAGE" then
            ripExtendsLeft = ripExtendsLeft - 1
        end
    end
end

clawPanel:SetScript("OnUpdate", function(self, elapsed)
    lastAbilityCalculation = lastAbilityCalculation + elapsed;
    if lastAbilityCalculation >= Claw.frequency then
		if UnitCanAttack("player", "target") and not UnitInVehicle("player") then
			if not hostileTargetted then
				clawPanel:EnableMouse(not Claw.locked)
				clawPreviousPanel:Hide()
				clawCDPanel:Hide()
				for i = 1, #targetHealthArray do targetHealthArray[i] = UnitHealth("target") end
				hostileTargetted = true
			else
				targetHealthArray[#targetHealthArray + 1] = UnitHealth("target")
				table.remove(targetHealthArray, 1)
			end
			oldAbilityNow = abilityNow
			oldAbilityCD = abilityCD
			currentTime = GetTime()
			abilityNow = CalculateBestChoiceAbility()
			if Claw.glow then
				abilityNow:glow(true)
				abilityCD:glow(true)
			end
			if abilityNow ~= oldAbilityNow then
				clawPanel.icon:SetTexture(abilityNow.icon)
				if abilityNow.icon then
					clawPanel.icon:Show()
					clawPanel.border:Show()
				else
					clawPanel.icon:Hide()
					clawPanel.border:Hide()
				end
				if Claw.glow then
					oldAbilityNow:glow(false)
				end
			end
			if abilityCD ~= oldAbilityCD then
				clawCDPanel.icon:SetTexture(abilityCD.icon)
				if abilityCD.icon then
					clawCDPanel:Show()
				else
					clawCDPanel:Hide()
				end
				if Claw.glow then
					oldAbilityCD:glow(false)
				end
			end
		elseif hostileTargetted then
			if Claw.glow then
				abilityNow:glow(false)
				abilityCD:glow(false)
			end
			abilityNow = NoSpell
			abilityCD = NoSpell
			clawPanel.icon:Hide()
			clawPanel.border:Hide()
			clawPanel:EnableMouse(false)
			clawPreviousPanel:Hide()
			clawCDPanel:Hide()
			for i = 1, #targetHealthArray do targetHealthArray[i] = 0 end
			hostileTargetted = false
		end
        lastAbilityCalculation = 0
    end
end)

clawPanel:SetScript("OnEvent", function(self, event, ...) events[event](self, ...) end)
for k in pairs(events) do
    clawPanel:RegisterEvent(k)
end

function SlashCmdList.Claw(msg, editbox)
    msg = { strsplit(" ", strlower(msg)) }
    if msg[1] == "locked" then
        if msg[2] then
			Claw.locked = msg[2] == "on" 
			clawPanel:EnableMouse(not Claw.locked and clawPanel.icon:IsVisible())
			clawPreviousPanel:EnableMouse(not Claw.locked)
			clawCDPanel:EnableMouse(not Claw.locked)
		end
        print("Claw - Locked: " .. (Claw.locked and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "scale" then
        if msg[2] then
			Claw.scale = tonumber(msg[2]) or 1
			clawPanel:SetScale(Claw.scale)
			clawPreviousPanel:SetScale(Claw.scale)
			clawCDPanel:SetScale(Claw.scale)
		end
		print("Claw - Scale: |cFFFFD000" .. Claw.scale .. "|r times")
	elseif msg[1] == "frequency" then
		Claw.frequency = tonumber(msg[2]) or 0.05
		print("Claw - Calculation frequency: Every |cFFFFD000" .. Claw.frequency .. "|r seconds")
	elseif msg[1] == "latency" then
		Claw.latency = tonumber(msg[2]) or 0.3
		print("Claw - Player reaction time: |cFFFFD000" .. Claw.latency .. "|r seconds")
    elseif msg[1] == "glow" then
        if msg[2] then
			Claw.glow = msg[2] == "on"
			if Claw.glow then
				abilityNow:glow(true)
				abilityCD:glow(true)
			else
				abilityNow:glow(false)
				abilityCD:glow(false)
			end
		end
        print("Claw - Glowing ability buttons: " .. (Claw.glow and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "previous" then
        if msg[2] then Claw.previous = msg[2] == "on" end
        print("Claw - Previous ability icon: " .. (Claw.previous and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "cd" then
        if msg[2] then Claw.cd = msg[2] == "on" end
        print("Claw - Cooldown ability icon: " .. (Claw.cd and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "misseffect" then
        if msg[2] then Claw.misseffect = msg[2] == "on" end
        print("Claw - Red border around previous ability when it fails to hit: " .. (Claw.misseffect and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "cat" then
        if msg[2] then Claw.cat = msg[2] == "on" end
        print("Claw - Show Claw UI in cat (feral) specialization: " .. (Claw.cat and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "bear" then
        if msg[2] then Claw.bear = msg[2] == "on" end
        print("Claw - Show Claw UI in bear (guardian) specialization: " .. (Claw.bear and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "bossonly" then
        if msg[2] then Claw.bossonly = msg[2] == "on" end
        print("Claw - Only use cooldowns on bosses: " .. (Claw.bossonly and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "ff" then
        if msg[2] then Claw.ff = msg[2] == "on" end
        print("Claw - Faerie Fire on non-boss mobs (Bear and Cat): " .. (Claw.ff and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "tfberserk" then
        if msg[2] then Claw.tfberserk = msg[2] == "on" end
        print("Claw - Tiger's Fury is macro'd to Berserk (Cat only): " .. (Claw.tfberserk and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "motw" then
        if msg[2] then Claw.motw = msg[2] == "on" end
        print("Claw - Mark of the Wild: " .. (Claw.motw and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "front" then
        if msg[2] then Claw.front = msg[2] == "on" end
        print("Claw - In front of enemy (Mangle replaces Shred): " .. (Claw.front and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "reset" then
		clawPanel:ClearAllPoints()
        clawPanel:SetPoint("CENTER", 0, -169)
		clawPreviousPanel:ClearAllPoints()
		clawPreviousPanel:SetPoint("BOTTOMRIGHT", clawPanel, "BOTTOMLEFT", -10, -5)
		clawCDPanel:ClearAllPoints()
		clawCDPanel:SetPoint("BOTTOMLEFT", clawPanel, "BOTTOMRIGHT", 10, -5)
		print("Claw - Position has been reset to default")
    else
        print("Claw (version: |cFFFFD000" .. GetAddOnMetadata("Claw", "Version") .. "|r) - Commands:")
        print("  /claw locked |cFF00C000on|r/|cFFC00000off|r - lock the Claw UI so that it can't be moved")
		print("  /claw scale |cFFFFD000[number]|r - set the scale of the Claw UI (default is 1 times)")
		print("  /claw frequency |cFFFFD000[number]|r - set the calculation frequency (default is every 0.05 seconds)")
		print("  /claw latency |cFFFFD000[number]|r - player's reaction time (default is 0.3 seconds)")
		print("  /claw glow |cFF00C000on|r/|cFFC00000off|r - glowing ability buttons on action bars")
		print("  /claw previous |cFF00C000on|r/|cFFC00000off|r - previous ability icon")
		print("  /claw cd |cFF00C000on|r/|cFFC00000off|r - cooldown ability icon (turn off for manual cooldown management)")
		print("  /claw misseffect |cFF00C000on|r/|cFFC00000off|r - red border around previous ability when it fails to hit")
		print("  /claw cat |cFF00C000on|r/|cFFC00000off|r - show Claw UI in cat (feral) specialization")
		print("  /claw bear |cFF00C000on|r/|cFFC00000off|r - show Claw UI in bear (guardian) specialization")
		print("  /claw bossonly |cFF00C000on|r/|cFFC00000off|r - only use cooldowns on bosses")
		print("  /claw ff |cFF00C000on|r/|cFFC00000off|r - use Faerie Fire (cat only)")
		print("  /claw tfberserk |cFF00C000on|r/|cFFC00000off|r - Tiger's Fury is macro'd to Berserk (cat only)")
		print("  /claw motw |cFF00C000on|r/|cFFC00000off|r - use Mark of the Wild")
		print("  /claw front |cFF00C000on|r/|cFFC00000off|r - in front of enemy (Mangle replaces Shred)")
        print("  /claw reset - reset the location of the Claw UI to default")
		if Claw_Resources then
			print("For Claw Resources commands, please type |cFFFFD000/cres|r.")
		end
    end
end
